/*
 * File   : platten.c
 * Purpose: This is the main file of the application platten and
 *          initializes all the resources used by the application.
 * Autor  : Andreas Theofilu
 *
 * Copyright (C) 1998, 1999, 2000 by Theos Soft.
 */

/* 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <Xm/Xm.h>
#include <X11/Shell.h>
#include <Xm/MwmUtil.h>
#include <Xm/Protocols.h>

#include <Xmt/Xmt.h>
#include <Xmt/Create.h>
#include <Xmt/Converters.h>
#include <Xmt/Procedures.h>
#include <Xmt/Layout.h>
#include <Xmt/Pixmap.h>
#include <Xmt/Xpm.h>
#include <Xmt/Icon.h>
#include <Xmt/Dialog.h>
#include <Xmt/Dialogs.h>
#include <Xmt/InputField.h>
#include <Xmt/Symbols.h>
#include <Xmt/WidgetType.h>
#include <Xmt/Include.h>
#include <Xbae/Matrix.h>
#include <stdlib.h>
#include <unistd.h>
#include <ctype.h>
#include "platten.h"
#include "config.h"

#include "platten.xpm"

/*
 * Global used Variables
 */
Widget toplevel;
application_resource_t app_resources;

/*
 * Some resources used by the program
 */
static XtResource ProgValues[] = {
	{ "Database", "Database", XmRString,
	   sizeof (String),
	   XtOffset (application_resource_t*, Database),
	   XmRString, "INFORMIX" },
	{ "Version", "Version", XmRString,
	  sizeof (String),
	  XtOffset (application_resource_t*, Version),
	  XmRString, "Unknown" },
	{ "datePattern", "datePattern", XmRString,
	  sizeof (String),
	  XtOffset (application_resource_t*, datePattern),
	  XmRString, "dd.mm.yyyy" },
	{ "CDDBPath", "CDDBPath", XmRString,
	  sizeof (String),
	  XtOffset (application_resource_t*, CDDBPath),
	  XmRString, "/usr/X11/lib/X11/xmcd/cddb" },
	{ "cdrom", "cdrom", XmRString,
	  sizeof (String),
	  XtOffset (application_resource_t*, cdrom),
	  XmRString, "/dev/cdrom" },
	{ "cdmount", "cdmount", XmRString,
	  sizeof (String),
	  XtOffset (application_resource_t*, cdmount),
	  XmRString, "/cdrom" },
	{ "helpCmd", "helpCmd", XmRString,
	  sizeof (String),
	  XtOffset (application_resource_t*, helpCmd),
	  XmRString, "xmhelp" },
	{ "iCurrDigits", "iCurrDigits", XtRInt,
	  sizeof (int),
	  XtOffset (application_resource_t*, iCurrDigits),
	  XtRInt, (XtPointer)2 },
	{ "iCurrency", "iCurrency", XtRInt,
	  sizeof (int),
	  XtOffset (application_resource_t*, iCurrency),
	  XtRInt, (XtPointer)0 },
	{ "iNegCurr", "iNegCurr", XtRInt,
	  sizeof (int),
	  XtOffset (application_resource_t*, iNegCurr),
	  XtRInt, (XtPointer)2 },
	{ "sCurrency", "sCurrency", XmRString,
	  sizeof (String),
	  XtOffset (application_resource_t*, sCurrency),
	  XmRString, "s" },
	{ "sDecimal", "sDecimal", XmRString,
	  sizeof (String),
	  XtOffset (application_resource_t*, sDecimal),
	  XmRString, "." },
	{ "sThousand", "sThousand", XmRString,
	  sizeof (String),
	  XtOffset (application_resource_t*, sThousand),
	  XmRString, "," }
};

static XmtProcedureInfo xt_procedures[] = {
	{"FirstFillCallback", (XmtProcedure)FirstFillCallback,
	 {XtRWidget, XtRWidget}},
	{"rclInitChooser", (XmtProcedure)rclInitChooser,
	 {XtRWidget, XtRWidget}},
	{"OptionDialogInit", (XmtProcedure)OptionDialogInit,
	 {XtRWidget}},
	{"UpdrclInitDiscid", (XmtProcedure)UpdrclInitDiscid,
	 {XtRWidget}},
	{"FirstSingleFillCallback", (XmtProcedure)FirstSingleFillCallback,
	 {XtRWidget, XtRWidget}},
	{"addSingleDisableButtonsCallback",
	 (XmtProcedure)addSingleDisableButtonsCallback,
	 {XtRWidget, XtRWidget}},
	{"FirstWunschFillCallback", (XmtProcedure)FirstWunschFillCallback,
	 {XtRWidget, XtRWidget}},
	{"GetHelp", (XmtProcedure)GetHelp,
	 {XtRWidget, XmRString}}
};

char *fallback_resource[] = {
	#include "plattenEN.ad.h"
	NULL
};

/*
 * Prototype function calls in this file not used in any other file.
 */
static void About ();
void Quit ();

/*
 * Entry point and start of the program :-).
 */
int main (int argc, char *argv[])
{
XtAppContext app;
XmtImage *platten_image;
Pixmap icon, mask;
Arg arglist[10];
int n = 0;

	XtSetArg (arglist[n], XmNiconName, "Platten"); n++;
	XtSetArg (arglist[n], XmNminHeight, 200); n++;
	XtSetArg (arglist[n], XmNminWidth, 400); n++;
	XtSetArg (arglist[n], XmNbaseHeight, 200); n++;
	XtSetArg (arglist[n], XmNbaseWidth, 400); n++;
	XtSetArg (arglist[n], XmNmaxHeight, 200); n++;
	XtSetArg (arglist[n], XmNmaxWidth, 400); n++;
	XtSetArg (arglist[n], XmNmwmDecorations, MWM_DECOR_BORDER |
		MWM_DECOR_MINIMIZE | MWM_DECOR_TITLE | MWM_DECOR_MENU); n++;
	XtSetArg (arglist[n], XmNmwmFunctions, MWM_FUNC_MOVE |
		MWM_FUNC_MINIMIZE | MWM_FUNC_CLOSE); n++;

	toplevel = XmtInitialize (&app, "Platten", NULL, 0, &argc, argv,
				  fallback_resource, arglist, n);

	GetResourcePath ("Init");
	GetResourcePath ("Menu");
	GetResourcePath ("Messages");
	XmtRegisterWidgetClass ("XbaeMatrix", xbaeMatrixWidgetClass);
	XmtAddDeleteCallback (toplevel, XmDO_NOTHING, Quit, NULL);

	XtGetApplicationResources (toplevel, (XtPointer)&app_resources,
			ProgValues, XtNumber (ProgValues), NULL, 0);

	if (!app_resources.Version || strcmp (app_resources.Version, VERSION))
	{
	   fprintf (stderr, "The resource file is for a different version (%s) of this program\nor even for another program!\nThis program will not work without the correct resource file.\n", app_resources.Version);
	   exit (1);
	}
#ifdef HAVE_PSQL
	app_resources.Database = "POSTGRESQL";
#else
	app_resources.Database = "INFORMIX";
#endif
	XmtRegisterAll ();
	XmtRegisterImprovedIcons (toplevel, NULL);
	XmtRegisterProcedures (xt_procedures, XtNumber (xt_procedures));

	XmtRegisterCallbackProcedure ("NewSingle", NewSingle, XmtRCallbackUnused);
	XmtRegisterCallbackProcedure ("SearchSingle", SearchSingle, XmtRCallbackUnused);
	XmtRegisterCallbackProcedure ("About", About, XmtRCallbackUnused);
	XmtRegisterCallbackProcedure ("Quit", Quit, XmtRCallbackUnused);
	XmtRegisterCallbackProcedure ("CheckDateCallback",
				CheckDateCallback, XmtRCallbackData);
	XmtRegisterCallbackProcedure ("CheckNumericCallback",
				CheckNumericCallback, XmtRCallbackData);
	XmtRegisterCallbackProcedure ("StripNumericCallback",
				StripNumericCallback, XmtRCallbackData);
	XmtRegisterCallbackProcedure ("PointNumericCallback",
				PointNumericCallback, XmtRCallbackData);
	XmtRegisterCallbackProcedure ("AddNewSingleCallback",
				AddNewSingleCallback, XmtRCallbackData);
	XmtRegisterCallbackProcedure ("ClearValuesCallback",
				ClearValuesCallback, XmtRCallbackData);
	XmtRegisterCallbackProcedure ("QuerySingleCallback",
				QuerySingleCallback, XmtRCallbackData);
	XmtRegisterCallbackProcedure ("siPartChooserCallback",
				siPartChooserCallback, XmtRCallbackData);
	XmtRegisterCallbackProcedure ("SingleSearchTypeCallback",
				SingleSearchTypeCallback, XmtRCallbackData);
	XmtRegisterCallbackProcedure ("UpdateSingleCallback",
				UpdateSingleCallback, XmtRCallbackData);
	XmtRegisterCallbackProcedure ("siEnterCellCallback",
				siEnterCellCallback, XmtRCallbackData);
	XmtRegisterCallbackProcedure ("OkaySingleCallback",
				OkaySingleCallback, XmtRCallbackData);
	
	XmtRegisterCallbackProcedure ("NewLP", NewLP, XmtRCallbackUnused);
	XmtRegisterCallbackProcedure ("NewLPCut", NewLPCut, XmtRCallbackUnused);
	XmtRegisterCallbackProcedure ("NewLPCutDialog", NewLPCutDialog, XmtRCallbackUnused);
	XmtRegisterCallbackProcedure ("UpdateLpCallback",
				UpdateLpCallback, XmtRCallbackData);
	XmtRegisterCallbackProcedure ("AddLpLabelCallback",
				AddLpLabelCallback, XmtRCallbackData);
	XmtRegisterCallbackProcedure ("AddLpCutsCallback",
				AddLpCutsCallback, XmtRCallbackData);
	XmtRegisterCallbackProcedure ("rclClearValuesCallback",
				rclClearValuesCallback, XmtRCallbackData);
	XmtRegisterCallbackProcedure ("rccClearValuesCallback",
				rccClearValuesCallback, XmtRCallbackData);
	XmtRegisterCallbackProcedure ("rclAutoCallback",
				rclAutoCallback, XmtRCallbackUnused);
	XmtRegisterCallbackProcedure ("SearchLps", SearchLps, XmtRCallbackUnused);
	XmtRegisterCallbackProcedure ("LPSearchValueChangedCallback",
				LPSearchValueChangedCallback, XmtRCallbackData);
	XmtRegisterCallbackProcedure ("MasterListCallback",
				MasterListCallback, XmtRCallbackData);
	XmtRegisterCallbackProcedure ("lpsPartChooserCallback",
				lpsPartChooserCallback, XmtRCallbackData);
	XmtRegisterCallbackProcedure ("lpsSearchTypeCallback",
				lpsSearchTypeCallback, XmtRCallbackData);
	XmtRegisterCallbackProcedure ("lpsEnterCellCallback",
				lpsEnterCellCallback, XmtRCallbackData);
	XmtRegisterCallbackProcedure ("lpsCutMasterCallback",
				lpsCutMasterCallback, XmtRCallbackData);
	XmtRegisterCallbackProcedure ("NewKennChangedCallback",
				NewKennChangedCallback, XmtRCallbackData);
	XmtRegisterCallbackProcedure ("UpdateKennChangedCallback",
				UpdateKennChangedCallback, XmtRCallbackData);
	XmtRegisterCallbackProcedure ("GetCDDBCallback",
				GetCDDBCallback, XmtRCallbackData);
	XmtRegisterCallbackProcedure ("PutCDDBCallback",
				PutCDDBCallback, XmtRCallbackData);
	XmtRegisterCallbackProcedure ("SubmitCDDBCallback",
				SubmitCDDBCallback, XmtRCallbackData);

	XmtRegisterCallbackProcedure ("ListWish", ListWish, XmtRCallbackUnused);
	XmtRegisterCallbackProcedure ("QueryWunschCallback",
				QueryWunschCallback, XmtRCallbackData);
	XmtRegisterCallbackProcedure ("wuPartChooserCallback",
				wuPartChooserCallback, XmtRCallbackData);
	XmtRegisterCallbackProcedure ("WunschSearchTypeCallback",
				WunschSearchTypeCallback, XmtRCallbackData);
	XmtRegisterCallbackProcedure ("wuEnterCellCallback",
				wuEnterCellCallback, XmtRCallbackData);
	XmtRegisterCallbackProcedure ("NewWish", NewWish, XmtRCallbackUnused);
	XmtRegisterCallbackProcedure ("AddNewWunschCallback",
				AddNewWunschCallback, XmtRCallbackData);
	XmtRegisterCallbackProcedure ("UpdateWunschCallback",
				UpdateWunschCallback, XmtRCallbackData);
	XmtRegisterCallbackProcedure ("DeleteWunschCallback",
				DeleteWunschCallback, XmtRCallbackData);
	
	XmtRegisterCallbackProcedure ("PrintSingles", PrintSingles, XmtRCallbackUnused);
	XmtRegisterCallbackProcedure ("prAllValueChangedCallback",
				prAllValueChangedCallback, XmtRCallbackData);
	XmtRegisterCallbackProcedure ("prtSearchVerifyCallback",
				prtSearchVerifyCallback, XmtRCallbackData);
	XmtRegisterCallbackProcedure ("StopPrintCallback",
				StopPrintCallback, XmtRCallbackData);

	XmtRegisterCallbackProcedure ("PrintWish", PrintWish, XmtRCallbackUnused);
	XmtRegisterCallbackProcedure ("prAllWishValueChangedCallback",
			prAllWishValueChangedCallback, XmtRCallbackData);
	XmtRegisterCallbackProcedure ("prtSearchWishVerifyCallback",
				prtSearchVerifyCallback, XmtRCallbackData);

	XmtRegisterCallbackProcedure ("PrintLps", PrintLps, XmtRCallbackUnused);
	XmtRegisterCallbackProcedure ("prAllLpsValueChangedCallback",
				prAllLpsValueChangedCallback, XmtRCallbackData);
	XmtRegisterCallbackProcedure ("prtSearchLpsVerifyCallback",
				prtSearchLpsVerifyCallback, XmtRCallbackData);

	XmtRegisterCallbackProcedure ("Options", Options, XmtRCallbackUnused);
	XmtRegisterCallbackProcedure ("OptionChooserChangedCallback",
				OptionChooserChangedCallback, XmtRCallbackData);

	XmtRegisterCallbackProcedure ("OpCDDB", OpCDDB, XmtRCallbackUnused);
	XmtRegisterCallbackProcedure ("opRemoteChooserCallback",
				opRemoteChooserCallback, XmtRCallbackData);
	XmtRegisterCallbackProcedure ("opProxyChooserCallback",
				opProxyChooserCallback, XmtRCallbackData);
	XmtRegisterCallbackProcedure ("AddCDDBServer",
				AddCDDBServer, XmtRCallbackData);
	XmtRegisterCallbackProcedure ("opLbServerCallback",
				opLbServerCallback, XmtRCallbackData);
	XmtRegisterCallbackProcedure ("opLbMailCallback",
				opLbMailCallback, XmtRCallbackData);
	XmtRegisterCallbackProcedure ("DeleteCDDBServer",
				DeleteCDDBServer, XmtRCallbackData);
	XmtRegisterCallbackProcedure ("AddEMAILDestination",
				AddEMAILDestination, XmtRCallbackData);
	XmtRegisterCallbackProcedure ("DeleteEMAILDestination",
				DeleteEMAILDestination, XmtRCallbackData);

	/*
	 * Registering the application icon.
	 */
	platten_image = XmtParseXpmData (platten);
	XmtRegisterImage ("platten", platten_image);
	icon = XmtLookupWidgetPixmap (toplevel, "platten");
	mask = XmtLookupBitmask (toplevel, "platten");
	XmtCreatePixmapIcon (toplevel, icon, mask);
	XtVaSetValues (toplevel, XmNiconPixmap, icon, NULL);

	/*
	 * Creating the Children of the main window and display all together.
	 */
	XmtCreateChildren (toplevel);
	XtRealizeWidget (toplevel);

	/*
	 * Initializing the program and the database.
	 */
	XmtDisplayBusyCursor (toplevel);

	if (!GetOptions (toplevel))
	   return -1;

	init ();
	XmtDisplayDefaultCursor (toplevel);
	XtAppMainLoop (app);
	return 0;
}

static void About ()
{
static Widget w = NULL;

	if (!w)
	{
	   GetResourcePath ("About");
	   w = XmtBuildDialog (toplevel, "about", NULL, 0);

	   XmtAddDeleteCallback (XmtGetShell (w), XmDO_NOTHING,
		XmtDialogCancelCallback, NULL);
	}
	else
	   XtManageChild (w);
}

void Quit ()
{
	CloseDatabase ();

	if (firstMail != NULL)
	{
	   struct MAIL *akt;

	   akt = firstMail;

	   while (akt)
	   {
	      akt = akt->next;
	      XtFree ((char *)firstMail);
	      firstMail = akt;
	   }

	   firstMail = NULL;
	}

	exit (0);
}

/*
 * Some Callbacks used by various dialog boxes
 */
Boolean GetResourcePath (char *name)
{
char *LANG, *path;
int len;

	LANG = getenv ("LANG");

	if (LANG == NULL)
	   LANG = "C";

	len = strlen (RESOURCE_PATH) + strlen (LANG) + strlen (name) + 6;

	if ((path = XtMalloc (len)) == NULL)
	{
	   XmtDisplayError (toplevel, "XtMallocError",
		"Not enough memory to handle LOCALE!\nUsing @fIfallback@fR resources instead.");
	   return False;
	}

	sprintf (path, "%s/%s/%s.ad", RESOURCE_PATH, LANG, name);

	if (access (path, R_OK) == 0)
	{
	   XmtLoadResourceFile (toplevel, path, False, True);
	   XtFree (path);
	   return True;
	}

	if (*LANG == 'C')	/* test for default LOCALE */
	{
	   XtFree (path);
	   return False;
	}

	/*
	 * Ok, we're still here, so no Resource was found. Now we set
	 * LANG explicitly to C. If there is still no Resource we must
	 * use the fallback resources.
	 */
	LANG = "C";

	sprintf (path, "%s/%s/%s.ad", RESOURCE_PATH, LANG, name);

	if (access (path, R_OK) == 0)
	{
	   XmtLoadResourceFile (toplevel, path, False, True);
	   XtFree (path);
	   return True;
	}

	XtFree (path);
	return False;
}

void CheckDateCallback (Widget w, XtPointer client_data, XtPointer call_data)
{
int ret;
XmtInputFieldCallbackStruct *data = (XmtInputFieldCallbackStruct *)call_data;

	ret = CheckDate (w, (char *)data->input);

	if (ret != 0)
	   data->okay = False;
	else
	   data->okay = True;
}

static char NumBuffer[120];

void CheckNumericCallback (Widget w, XtPointer client_data, XtPointer call_data)
{
int i, j, a;
char *p;
double zahl;
XmtInputFieldCallbackStruct *data = (XmtInputFieldCallbackStruct *)call_data;

	strcpy (&NumBuffer[0], data->input);

	if (StripNum (&NumBuffer[0]) == NULL)
	{
	   data->okay = False;
	   return;
	}

	j = strlen (NumBuffer);
	p = &NumBuffer[0];
	a = 0;

	for (i = 0; i < j; i++)
	{
	   if (!isdigit (*p) && *p != '.')
	   {
	      data->okay = False;
	      return;
	   }

	   if (*p == '.')
	      a++;

	   if (a > 1)
	   {
	      data->okay = False;
	      return;
	   }

	   p++;
	}

	data->okay = True;
	zahl = strtod (NumBuffer, NULL);

	if (Interpoint (zahl, 2, &NumBuffer[0], 0) == NULL)
	   return;

	data->input = NumBuffer;
}

void StripNumericCallback (Widget w, XtPointer client_data, XtPointer call_data)
{
String Data;

	Data = XmtInputFieldGetString (w);

	if (!strlen (Data))
	   return;

	strcpy (NumBuffer, Data);

	if (StripNum (&NumBuffer[0]) == NULL)
	   return;

	XmtInputFieldSetString (w, NumBuffer);
}

void PointNumericCallback (Widget w, XtPointer client_data, XtPointer call_data)
{
double zahl;
String Data;

	Data = XmtInputFieldGetString (w);
	NumBuffer[0] = 0;

	if (!strlen (Data))
	   return;

	zahl = strtod (Data, NULL);

	if (Interpoint (zahl, 2, &NumBuffer[0], 0) == NULL)
	   return;

	XmtInputFieldSetString (w, NumBuffer);
}
